#include "hmatrixfile.h"
#include "hfilter.h"
#include "hidealsmear.h"
// #include <TError.h>

using namespace std;

// ****************************************************************************
ClassImp(HFilter)

//-----------------------------------------------------------------------------
HFilter::HFilter(const HMatrixFile& file, const char* name) : 
	files(file), name(name), ptrS(0), openingAngle(0.)
{
}

//-----------------------------------------------------------------------------
HFilter::~HFilter()
{
   for_each( matrixArr.begin(), matrixArr.end(), DelObj() );
}

//-----------------------------------------------------------------------------
Bool_t HFilter::add(Int_t id, const char* name)
{
   if (0 == getMatrix(name))
   {
      matrixArr.push_back( new HMatrix(id, name, files) );
      return kTRUE;
   }
 return kFALSE;
}

//-----------------------------------------------------------------------------
const HMatrix* HFilter::getMatrix(const char* name) const
{
   list<HMatrix*>::const_iterator cIter = find_if( matrixArr.begin(), matrixArr.end(), bind2nd( FindName(), name ) );
   if (cIter == matrixArr.end()) {
      return 0;
   }
 return ( dynamic_cast<const HMatrix*>(*cIter) );
}

//-----------------------------------------------------------------------------
const HMatrix* HFilter::getMatrix(Int_t id, const char* type) const
{
   string tmp(type);
   list<HMatrix*>::const_iterator cIter;
   if ("acc" == tmp) {
      cIter = find_if( matrixArr.begin(), matrixArr.end(), bind2nd( FindAcc(), id ) );
   } else if ("eff" == tmp) {
      cIter = find_if( matrixArr.begin(), matrixArr.end(), bind2nd( FindEff(), id ) );
   } else if ("acceff" == tmp) {
      cIter = find_if( matrixArr.begin(), matrixArr.end(), bind2nd( FindAccEff(), id ) );
   }
   if (cIter == matrixArr.end()) {
      return 0;
   }
 return ( dynamic_cast<const HMatrix*>(*cIter) );
}


//-----------------------------------------------------------------------------
void HFilter::print() const
{
   list<HMatrix*>::const_iterator cIter;
   for (cIter = matrixArr.begin(); cIter != matrixArr.end(); ++cIter)
   {
      cout << "Matrix: " << (*cIter)->getName() << ", type of matrix: " << (*cIter)->getType() << endl;
   }
}

//-----------------------------------------------------------------------------
void HFilter::smear(string name) 
{
   if (ptrS) delete ptrS;
   ptrS = 0;
   if (name == "ideal") ptrS = new HIdealSmear;
}

//-----------------------------------------------------------------------------
void HFilter::filter(HParticle& ref) const
{
   int id = ref.getId();
   double mom = 1000. * ref.getP();
   double theta = ref.getTheta("deg");
   double phi = ref.getPhi("deg", "one sector");
   double phiall = ref.getPhi("deg") - 180.; // because all eff matrices are -pi, +pi so far
//   cout << "Zanim przefiltrujemy id = " << id << " mom,theta,phi: " << mom << ", " << theta << ", " << phiall << endl;
   const HMatrix* acceptance = getMatrix( id, "acc" );
   const HMatrix* efficiency = getMatrix( id, "eff" );
   if (acceptance != 0)
   {
      ref.setAcc( (*acceptance)(mom, theta, phi) );
   }
   if (efficiency != 0)
   {
      ref.setEff( (*efficiency)(mom, theta, phiall) );
   }
//   cout << "acc: " << ref.getAcc() << " eff: " << ref.getEff() << endl;
   if (ptrS) ptrS->smear( ref );
}

//-----------------------------------------------------------------------------
void HFilter::filter(HNParticle& ref, bool isMissing) const
{
   if (openingAngle > 0.0)
   {
      ref.setOAstat( true );
      int iStart = 0;
      if (isMissing == true) iStart = 2;
      
      // we scan all pairs with respect to their opening angle
      int size = ref.getSize();
      for (int i = iStart; i < size; ++i)
      {
         for (int j = i+1; j < size; ++j)
         {
	    if ( ref.getOA( i, j ) < openingAngle )
	       ref.setOAstat( false );
	 }
      }
   }
}





