#ifndef HFILTER_H
#define HFILTER_H

#include <TROOT.h>
#include <TFile.h>
#include <string>
#include <list>
#include <functional>
#include <algorithm>
#include "hmatrix.h"
#include "hparticle.h"
#include "hnparticle.h"
#include "hsmear.h"

class HMatrixFile;

// ****************************************************************************
class HFilter
{

    struct DelObj {
       void operator()(HMatrix* ptr) {
          delete ptr;
       }
    };

    struct FindName : std::binary_function<HMatrix*, const char*, bool> {
       bool operator()(HMatrix* ptrM, const char* name) const {
          return ( std::string(name) == ptrM->getName() );
       }
    };

    struct FindAcc : std::binary_function<HMatrix*, Int_t, bool> {
       bool operator()(HMatrix* ptrM, Int_t id) const {
          return ( id == ptrM->getId() && std::string("acc") == ptrM->getType() );
       }
    };

    struct FindEff : std::binary_function<HMatrix*, Int_t, bool> {
       bool operator()(HMatrix* ptrM, Int_t id) const {
          return ( id == ptrM->getId() && std::string("eff") == ptrM->getType() );
       }
    };

    struct FindAccEff : std::binary_function<HMatrix*, Int_t, bool> {
       bool operator()(HMatrix* ptrM, Int_t id) const {
          return ( id == ptrM->getId() && std::string("acceff") == ptrM->getType() );
       }
    };

public:

   HFilter(const HMatrixFile& file, const char* name = "");
   ~HFilter();

   Bool_t add(Int_t id, const char* name);
   const HMatrix* getMatrix(const char* name) const;
   const HMatrix* getMatrix(Int_t id, const char* type) const;

   void filter(HParticle& ref) const;
   void filter(HNParticle& ref, bool isMissing = false) const;

   void print() const;

   void smear(std::string name);

   void openingangle(double angle) { openingAngle = angle; }

   std::string getName() const { return name; }

private:

   const HMatrixFile& files;
   std::list<HMatrix*> matrixArr;
   HSmear *ptrS;
   double openingAngle;
   
   // std::list<HMatrix*>::iterator iterMatrixArr;
   
   std::string name;

ClassDef(HFilter, 0)
};
// ****************************************************************************

#endif // HFILTER_H
