#include "hidealsmear.h"
#include <TError.h>
#include <TMath.h>


// ****************************************************************************
ClassImp(HIdealSmear)


//-----------------------------------------------------------------------------
void HIdealSmear::smear(HParticle& ref)
{
  // smearing of momentum due to the HADES resolution
  // n == 2 : MDC I, II
  // n == 3 : MDC I, II, III
  // n == 4 : MDC I, II, III, IV

  int n = 4;

  Double_t mom = ref.getP();
  Double_t theta = ref.getTheta();
  Double_t phi = ref.getPhi();
  Double_t sin_theta = TMath::Sin( theta );
  Double_t gamma;
  switch (n)
  {
     case 2: gamma = 0.01*mom*(2.0 + 13.63 * mom);
             break;
     case 3: gamma = 0.01*mom*(1.0 + 3.6 * mom);
             break;
     case 4: gamma = 0.01*mom*(0.5 * 1.36 * mom);
             break;
     default: gamma = 0.1;
              break;
  }
  if (TMath::Abs(sin_theta)<1.e-6) sin_theta = (sin_theta>0.) ? 1.e-6 : -1.e-6;
  Double_t smeared_mom = TMath::Abs( REngine.Gaus(mom, gamma ) );
  Double_t smeared_theta = REngine.Gaus(theta, 0.0023 );
  Double_t smeared_phi = ( REngine.Gaus(sin_theta*phi, 0.0023 ) )/sin_theta;
  Double_t theta_check = smeared_theta-2.*kPI*static_cast<int>(smeared_theta/(2.*kPI));
  if (theta_check>kPI||theta_check<0.) smeared_phi -= kPI;
  mom = smeared_mom;
  theta = smeared_theta;
  phi = smeared_phi;
  // -------------------------- recalculate momx, momy, momz -------------
  Double_t momx = mom * TMath::Sin(theta) * TMath::Cos(phi);
  Double_t momy = mom * TMath::Sin(theta) * TMath::Sin(phi);
  Double_t momz = mom * TMath::Cos(theta);
  ref.setPxPyPz( momx, momy, momz );

}



