#include "hinvariantfactory.h"
#include "hpidgeanttrackset.h"
#include "hgeantkine.h" 
#include "PParticle.h"
#include <TMath.h>
#include <TLorentzVector.h>

using namespace std;

// ****************************************************************************
ClassImp(HInvariantFactory)

//-----------------------------------------------------------------------------
HInvariantFactory::HInvariantFactory(HParticlePool& ref, std::string name) : HPoolFactory( ref, name )
{
}

//-----------------------------------------------------------------------------
HInvariantFactory::~HInvariantFactory()
{
}


//-----------------------------------------------------------------------------
void HInvariantFactory::add(EParticle p1)
{
   particleNum[ p1 ]++;
   particleComb[ p1 ] = 0;
   prongNumber = 1;
}

//-----------------------------------------------------------------------------
void HInvariantFactory::add(EParticle p1, EParticle p2)
{
   particleNum[ p1 ]++;
   particleNum[ p2 ]++; 
   particleComb[ p1 ] = 0;
   particleComb[ p2 ] = 0;
   prongNumber = 2;
}

//-----------------------------------------------------------------------------
void HInvariantFactory::add(EParticle p1, EParticle p2, EParticle p3)
{
   particleNum[ p1 ]++;
   particleNum[ p2 ]++;
   particleNum[ p3 ]++; 
   particleComb[ p1 ] = 0;
   particleComb[ p2 ] = 0;
   particleComb[ p3 ] = 0;
   prongNumber = 3;
}

//-----------------------------------------------------------------------------
void HInvariantFactory::add(EParticle p1, EParticle p2, EParticle p3, EParticle p4)
{
   particleNum[ p1 ]++;
   particleNum[ p2 ]++;
   particleNum[ p3 ]++;
   particleNum[ p4 ]++; 
   particleComb[ p1 ] = 0;
   particleComb[ p2 ] = 0;
   particleComb[ p3 ] = 0;
   particleComb[ p4 ] = 0;
   prongNumber = 4;
}


//-----------------------------------------------------------------------------
void HInvariantFactory::combine()
{
   int i1, i2, i3, i4, repetitionNr, combinitaionNr;
   EParticle pid;

   particleSet.clear();
   count();
   particleSet.resize( number );
   // here all HParticle objects from HParticlePool should be combined according to pattern from mapN

   MultiParticleIter mpIter;
   MultiParticleIter mpBeginIter;
   for (particleNumIter = particleComb.begin(); particleNumIter != particleComb.end(); ++particleNumIter)
   // loop over all particle species
   {
      pid = particleNumIter->first;
      combinitaionNr = particleNumIter->second;
      if (combinitaionNr > 0) // for given particle: if number of combinations > 0
      {
            repetitionNr = number / combinitaionNr;
            for (int k=0; k < combinitaionNr; ++k) // all combinations
            {
               switch (particleNum[ pid ])
               {
                   case 1: i1 = ptrFH->map1[ 10 + refPool.particleNum[ pid ] ][ k ][0];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           break;

                   case 2: i1 = ptrFH->map2[ 20 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map2[ 20 + refPool.particleNum[ pid ] ][ k ][1];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           break;

                   case 3: i1 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][1];
                           i3 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][2];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i3--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           break;

                   case 4: i1 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][1];
                           i3 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][2];
                           i4 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][3];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i3--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           mpIter = mpBeginIter;
                           while (i4--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] += mpIter->second;
                           break;

               }
            }
      }
   }

}

