// This macro filters data
// Date: 17.11.05 by WP

#include "hmatrixhistogram.h"


// ****************************************************************************
ClassImp(HMatrixHistogram)



//-----------------------------------------------------------------------------
void HMatrixHistogram::setStatus(Bool_t stat)
{
  status = stat; 
}


//-----------------------------------------------------------------------------
Bool_t HMatrixHistogram::getStatus() const 
{
   return status;
}


//-----------------------------------------------------------------------------
Bool_t HMatrixHistogram::setHisto()
{
   if (file)
   {
      hist = NULL;
      if (this->GetName())
      {
         //cout << " HISTOGRAM NAME: "<<this->GetName()<<endl;
         hist = (TH1*)file->Get(this->GetName());
         setStatus(kTRUE);
      }
      else
      {
         Error("setHisto",
               "No histogram name defined for file '%s'.",
               file->GetName());
         setStatus(kFALSE);
      }

      if (hist && getSize() == 0)
      {
         Error("setHisto",
               "No histogram axes names defined for histogram '%s'.",
               hist->GetName());
         setStatus(kFALSE);
      }
      else
      {
         if (getSize() == 1)
         {
            lower_limit[0] = ((TH1F*)hist)->GetXaxis()->GetXmin();
            upper_limit[0] = ((TH1F*)hist)->GetXaxis()->GetXmax();
         }
         else if (getSize() == 2)
         {
            lower_limit[0] = ((TH2F*)hist)->GetXaxis()->GetXmin();
            upper_limit[0] = ((TH2F*)hist)->GetXaxis()->GetXmax();
            lower_limit[1] = ((TH2F*)hist)->GetYaxis()->GetXmin();
            upper_limit[1] = ((TH2F*)hist)->GetYaxis()->GetXmax();
         }
         else if (getSize() == 3)
         {
            lower_limit[0] = ((TH3F*)hist)->GetXaxis()->GetXmin();
            upper_limit[0] = ((TH3F*)hist)->GetXaxis()->GetXmax();
            lower_limit[1] = ((TH3F*)hist)->GetYaxis()->GetXmin();
            upper_limit[1] = ((TH3F*)hist)->GetYaxis()->GetXmax();
            lower_limit[2] = ((TH3F*)hist)->GetZaxis()->GetXmin();
            upper_limit[2] = ((TH3F*)hist)->GetZaxis()->GetXmax();
         }
      }

      if (hist == NULL)
      {
         Error("setHisto",
               "Histogram '%s' not found in file '%s'",
               this->GetName(), file->GetName());
         setStatus(kFALSE);
      }


   }
   else
   {
      Error("setHisto", "File with histogram not set.");
      setStatus(kFALSE);
   }

return getStatus();
}



//-----------------------------------------------------------------------------
Int_t HMatrixHistogram::getSize()
{
   if (size == 0)
   {
      calcSize();
   }

return size;
}


//-----------------------------------------------------------------------------
Int_t HMatrixHistogram::calcSize()
{
   size = 0;
   if (xname.Length() > 0) size++;
   if (yname.Length() > 0) size++;
   if (zname.Length() > 0) size++;

return size;
}


//-----------------------------------------------------------------------------
void HMatrixHistogram::setXYZunit(const char* x, const char* y, const char* z)
{
   xunit = x;
   yunit = y;
   zunit = z;

   valscale[0] = valscale[1] = valscale[2] = 1.;

   switch (mapping[0])
   {
      case 0: if (xunit == "GeV")
              {
                 valscale[0] = 0.001;
              }
              else if (xunit == "keV")
              {
                 valscale[0] = 1000.;
              }
              break;
      case 1: if (yunit == "GeV")
              {
                 valscale[1] = 0.001;
              }
              else if (yunit == "keV")
              {
                 valscale[1] = 1000.;
              }
              break;
      case 2: if (zunit == "GeV")
              {
                 valscale[2] = 0.001;
              }
              else if (zunit == "keV")
              {
                 valscale[2] = 1000.;
              }
              break;
   }

   switch (mapping[1])
   {
      case 0: if (xunit == "rad")
              {
                 valscale[0] = TMath::Pi()/180.;
              }
              break;
      case 1: if (yunit == "rad")
              {
                 valscale[1] = TMath::Pi()/180.;
              }
              break;
      case 2: if (zunit == "rad")
              {
                 valscale[2] = TMath::Pi()/180.;
              }
              break;
   }

   switch (mapping[2])
   {
      case 0: if (xunit == "rad")
              {
                 valscale[0] = TMath::Pi()/180.;
              }
              break;
      case 1: if (yunit == "rad")
              {
                 valscale[1] = TMath::Pi()/180.;
              }
              break;
      case 2: if (zunit == "rad")
              {
                 valscale[2] = TMath::Pi()/180.;
              }
              break;
   }
}


//-----------------------------------------------------------------------------
void HMatrixHistogram::setVal(Float_t value)
{
   val[mapping[0]] = value * valscale[mapping[0]];
}


//-----------------------------------------------------------------------------
void HMatrixHistogram::setVal(Float_t mom, Float_t theta)
{
   val[mapping[0]] = mom * valscale[mapping[0]];
   val[mapping[1]] = theta * valscale[mapping[1]];
}   

//-----------------------------------------------------------------------------
void HMatrixHistogram::setVal(Float_t mom, Float_t theta, Float_t phi)
{
   val[mapping[0]] = mom * valscale[mapping[0]];
   val[mapping[1]] = theta * valscale[mapping[1]];
   val[mapping[2]] = phi * valscale[mapping[2]];
}   




//-----------------------------------------------------------------------------
void HMatrixHistogram::setXYZname(const char* x, const char* y, const char* z)
{
   xname = x;
   yname = y;
   zname = z;

   mapping[0] = 0;
   mapping[1] = 1;
   mapping[2] = 2;

   if (xname=="mom")
   {
      mapping[0] = 0;
   }
   else if (yname=="mom")
   {
      mapping[0] = 1;
   }
   else if (zname=="mom")
   {
      mapping[0] = 2;
   }

   if (xname=="theta")
   {
      mapping[1] = 0;
   }
   else if (yname=="theta")
   {
      mapping[1] = 1;
   }
   else if (zname=="theta")
   {
      mapping[1] = 2;
   }
  
   if (xname=="phi")
   {
      mapping[2] = 0;
   }
   else if (yname=="phi")
   {
      mapping[2] = 1;
   }
   else if (zname=="phi")
   {
      mapping[2] = 2;
   }


}

//-----------------------------------------------------------------------------
Float_t HMatrixHistogram::getBinContent(Float_t mom, Float_t theta, Float_t phi) 
{
   setVal(mom, theta, phi);

   if (val[mapping[1]] > lower_limit[mapping[1]] &&
       val[mapping[1]] < upper_limit[mapping[1]] &&
       val[mapping[2]] > lower_limit[mapping[2]] &&
       val[mapping[2]] < upper_limit[mapping[2]])
   {
       if (val[mapping[0]] > lower_limit[mapping[0]])
       {
           if (val[mapping[0]] < upper_limit[mapping[0]])
           {
              return hist->GetBinContent(hist->FindBin(val[0], val[1], val[2]));
           }
           else
           {
              val[mapping[0]] -= 0.0001; // I want the last mom bin value
              return ((TH3F*)hist)->GetBinContent(((TH3F*)hist)->FindBin(val[0], val[1], val[2]));
           }
       }
   }
return 0.;
}


//-----------------------------------------------------------------------------
Float_t HMatrixHistogram::getBinContent(Float_t value) 
{
   setVal(value);

   if (val[mapping[0]] > lower_limit[mapping[0]])
   {
      if (val[mapping[0]] < upper_limit[mapping[0]])
      {
         return hist->GetBinContent(hist->FindBin(val[0]));
      }
      else
      {
         val[mapping[0]] -= 0.0001; // I want the last bin value
         return ((TH1F*)hist)->GetBinContent(((TH1F*)hist)->FindBin(val[0]));
      }
   }
return 0.;
}



//-----------------------------------------------------------------------------
Float_t HMatrixHistogram::getBinContent(Float_t mom, Float_t theta) 
{
   setVal(mom, theta);

  if (val[mapping[1]] > lower_limit[mapping[1]] &&
       val[mapping[1]] < upper_limit[mapping[1]])
  {

   if (val[mapping[0]] > lower_limit[mapping[0]])
   {
      if (val[mapping[0]] < upper_limit[mapping[0]])
      {
         return hist->GetBinContent(hist->FindBin(val[0]));
      }
      else
      {
         val[mapping[0]] -= 0.0001; // I want the last bin value
         return ((TH1F*)hist)->GetBinContent(((TH1F*)hist)->FindBin(val[0]));
      }
   }

  }

return 0.;
}




//-----------------------------------------------------------------------------
HMatrixHistogram::HMatrixHistogram() 
{
   HMatrixHistogram("default");
}


//-----------------------------------------------------------------------------
HMatrixHistogram::HMatrixHistogram(const char* name) : TNamed(name,"")
{
   file = NULL;
   size = 0;
   hist = NULL;
   xname = "";
   yname = "";
   zname = "";
   xunit = "";
   yunit = "";
   zunit = "";
   mapping[0] = 0;
   mapping[1] = 1;
   mapping[2] = 2;
   val[0] = 0.;
   val[1] = 0.;
   val[2] = 0.;
   valscale[0] = 0.;
   valscale[1] = 0.;
   valscale[2] = 0.;
   lower_limit[0] = 0.;
   lower_limit[1] = 0.;
   lower_limit[2] = 0.;
   upper_limit[0] = 0.;
   upper_limit[1] = 0.;
   upper_limit[2] = 0.;
   status = kFALSE;
}


HMatrixHistogram::~HMatrixHistogram()
{
}

//--------------------------------------------------------------
void HMatrixHistogram::setFile(TFile* pname)
{
   if (pname)
   {
      file = pname;
   }
   else
   {
      setStatus(kFALSE);
   }
}




