// This macro filters data
// Date: 17.11.05 by WP

#include "hmatrixparticle.h"
#include "hpidgeanttrackset.h"
#include <TMath.h>
#include <TLorentzVector.h>


// ****************************************************************************
ClassImp(HMatrixParticle)

HMatrixParticle::HMatrixParticle()
{
   HMatrixParticle(0, "none");
}

HMatrixParticle::HMatrixParticle(const Int_t part, const char* name) : TNamed(name,"")
{
   id = part;
   mom = 0.;
   theta = 0.;
   phi = 0.;
   momx = 0.;
   momy = 0.;
   momz = 0.;
   momt = 0.;
   rapidity = 0.;
   parent = 0;
   parentid = 0;
   grandparentid = 0;
   grandgrandparentid = 0;
   medium = 0;
   mechanism = 0;
   geninfo = 0.;
   geninfo1 = 0.;
   geninfo2 = 0.;
   acc = 0.;
   eff = 0.;
   pi = TMath::Pi();
   deg2rad = TMath::Pi()/180.;
   rad2deg = 180./TMath::Pi();
   switch (part)
   {
       case 2: myName = "ep";
               invmass = 0.511;
			   break;
       case 3: myName = "em";
               invmass = 0.511;
			   break;
       case 8: myName = "pip";
               invmass = 139.569;
			   break;
       case 9: myName = "pim";
               invmass = 139.569;
			   break;
       case 14: myName = "p";
               invmass = 938.272;
			   break;
   }
}

HMatrixParticle::~HMatrixParticle()
{
}

const char* HMatrixParticle::getName() const
{
   return myName.c_str();
}

TLorentzVector* HMatrixParticle::getLVector()
{
   return &vPart;
}

void HMatrixParticle::setId(const Int_t val)
   {
      id = val;
   }

void HMatrixParticle::setAcc(Float_t ac)
   {
      acc = ac;
   }

void HMatrixParticle::setEff(Float_t ef)
   {
      eff = ef;
   }

void HMatrixParticle::setXYZmom(const Float_t px, const Float_t py, const Float_t pz)
   {
      momx = px;
      momy = py;
      momz = pz;
   }

void HMatrixParticle::setMomThetaPhi(const Float_t p, const Float_t th, const Float_t ph)
   {
      mom = p;
      theta = th;
      phi = ph;
   }

void HMatrixParticle::setMomtRapidity(Float_t pt, const Float_t y)
   {
      momt = pt;
      rapidity = y;
   }

void HMatrixParticle::setParMedMech(const Int_t par, const Int_t med, const Int_t mech)
   {
      parent = par;
      medium = med;
      mechanism = mech;
   }

void HMatrixParticle::setParGparGGparId(const Int_t parid, const Int_t gparid, const Int_t ggparid)
{
   parentid = parid;
   grandparentid = gparid;
   grandgrandparentid = ggparid;
}

void HMatrixParticle::setGenInfo(const Float_t g0, const Float_t g1, const Float_t g2)
   {
      geninfo = g0;
      geninfo1 = g1;
      geninfo2 = g2;
   }

Int_t HMatrixParticle::getId() const 
   {
      return id;
   }

Float_t HMatrixParticle::getAcc() const 
   {
      return acc;
   }

Float_t HMatrixParticle::getEff() const 
   {
      return eff;
   }

Float_t HMatrixParticle::getMass() const 
   {
	   switch (id)
	   {
		  case 2: return TMath::Sqrt(mom*mom+0.511*0.511);
		  case 3: return TMath::Sqrt(mom*mom+0.511*0.511);
		  case 7: return TMath::Sqrt(mom*mom+134.976*134.976);
		  case 8: return TMath::Sqrt(mom*mom+139.569*139.569);
		  case 9: return TMath::Sqrt(mom*mom+139.569*139.569);
		  case 13: return TMath::Sqrt(mom*mom+939.565*939.565);
		  case 14: return TMath::Sqrt(mom*mom+938.272*938.272);
	   }
   return 0;
   }

Float_t HMatrixParticle::getMomx() const 
   {
      return momx;
   }

Float_t HMatrixParticle::getMomy() const 
   {
      return momy;
   }

Float_t HMatrixParticle::getMomz() const 
   {
      return momz;
   }

Float_t HMatrixParticle::getMom() const 
   {
      return mom;
   }

Float_t HMatrixParticle::getPt() const 
   {
      return momt;
   }


Float_t HMatrixParticle::getTheta() const 
   {
      return theta;
   }

Float_t HMatrixParticle::getPhi() const 
   {
      return phi;
   }

Int_t HMatrixParticle::getParentId() const 
   {
      return parentid;
   }

Int_t HMatrixParticle::getGrandparentId() const 
   {
      return grandparentid;
   }

Int_t HMatrixParticle::getGrandgrandparentId() const 
   {
      return grandgrandparentid;
   }

Float_t HMatrixParticle::getY() const 
   {
          return vPart.Rapidity();
   }

Int_t HMatrixParticle::getParent() const 
   {
      return parent;
   }

Int_t HMatrixParticle::getMedium() const 
   {
      return medium;
   }

Int_t HMatrixParticle::getMechanism() const 
   {
      return mechanism;
   }

Float_t HMatrixParticle::getGenInfo() const 
   {
      return geninfo;
   }

Float_t HMatrixParticle::getGenInfo1() const 
   {
      return geninfo1;
   }

Float_t HMatrixParticle::getGenInfo2() const 
   {
      return geninfo2;
   }


Bool_t HMatrixParticle::getUrQMDcondition() const
{
 return (medium == 18 && (mechanism == 5 || mechanism == 6));
}


Bool_t HMatrixParticle::getPLUTOcondition() const 
{
 return (parent == 0);
}


//--------------------------------------------------------------
Bool_t HMatrixParticle::operator=(HGeantKine& hit)
{
 if (hit.getID() == id)
 {
	HGeantKine *phit = 0;
    hit.getMomentum(vec);
    mom = hit.getTotalMomentum();
    momx = vec.getX();
    momy = vec.getY();
    momz = vec.getZ();
	momt = hit.getTransverseMomentum();
    theta = TMath::ACos(momz / mom) * TMath::RadToDeg();
    phi = TMath::ACos(momx/TMath::Sqrt(momx*momx+momy*momy));
    if (momy<0) phi = 2.*TMath::Pi()-phi;
    phi *= TMath::RadToDeg();
    hit.getCreator(parent, medium, mechanism);
    hit.getGenerator(geninfo, geninfo1, geninfo2);
	vPart.SetPxPyPzE(momx, momy, momz, getMass());
	rapidity = vPart.Rapidity();

    //HGeantKine* pKineParent = HPidGeantTrackSet::getGeantKine(parent, pGeantKineCat);
    //if(pKineParent!=NULL)
    //{
    //   pKineParent->getParticle(parent, parentid);
    //}

  return kTRUE;
 }
 return kFALSE;
}



//=======================================================


   Double_t detrk4[7][5] = {3.04669e-06, 2.92245e-06, 2.88894e-06, 2.54182e-06, 1.79336e-06,
                 3.45534e-06, 3.52212e-06, 3.43443e-06, 3.3966e-06,  3.31397e-06,
                 5.01234e-06, 5.46206e-06, 5.25006e-06, 5.06055e-06, 4.87875e-06,
                 6.29485e-06, 7.83364e-06, 8.64567e-06, 7.44434e-06, 6.44697e-06,
                 5.90814e-06, 7.49372e-06, 8.44064e-06, 7.33158e-06, 5.8983e-06 ,
                 5.00201e-06, 6.82347e-06, 8.42634e-06, 6.64071e-06, 4.32283e-06,
                 4.57113e-06, 7.85962e-06, 9.27056e-06, 7.13838e-06, 4.2649e-06 };

   Double_t multscrk4[7][5] = {0.00712136, 0.00774792, 0.00745848, 0.0072562, 0.0077318,
                 0.00947987, 0.00919469, 0.00923363, 0.0090577, 0.0088233,
                 0.0104807 , 0.0105168 , 0.0114445 , 0.0109526, 0.0103268,
                 0.0122493 , 0.013979  , 0.0148295 , 0.0142261, 0.0125651,
                 0.0132593 , 0.0151324 , 0.0171913 , 0.0157784, 0.0133762,
                 0.0127184 , 0.0166672 , 0.0197425 , 0.0166028, 0.0127183,
                 0.0121717 , 0.0187349 , 0.0235083 , 0.0183666, 0.0114857};

   Double_t detrk3[7][5] = {5.08616e-06, 5.57472e-06, 5.90756e-06, 5.30146e-06, 5.12577e-06,
                8.64475e-06, 8.44705e-06, 8.58983e-06, 7.7759e-06 , 6.88931e-06,
                1.23261e-05, 1.42034e-05, 1.55287e-05, 1.31425e-05, 1.10076e-05,
                1.75029e-05, 2.59217e-05, 2.72391e-05, 2.46139e-05, 1.84151e-05,
                1.75554e-05, 2.25547e-05, 2.5371e-05 , 2.30453e-05, 1.8036e-05 ,
                1.5159e-05 , 2.14014e-05, 2.37335e-05, 2.17028e-05, 1.56393e-05,
                1.70025e-05, 2.42703e-05, 2.65468e-05, 2.33554e-05, 1.5726e-05  };

   Double_t multscrk3[7][5] = {0.00705671, 0.00707791, 0.00719795, 0.00686656, 0.00468438,
                   0.00791494, 0.00971418, 0.00922887, 0.00921739, 0.00882323,
                   0.0101972 , 0.0108932 , 0.0109951 , 0.0111292 , 0.0103408 ,
                   0.0136591 , 0.0147924 , 0.0152897 , 0.0153152 , 0.0135126 ,
                   0.014621  , 0.0166669 , 0.0175123 , 0.0176646 , 0.0157295 ,
                   0.0146804 , 0.0180094 , 0.0183208 , 0.0172132 , 0.014879  ,
                   0.0130367 , 0.018929  , 0.0221153 , 0.0189616 , 0.0147333    };

   Double_t detkick[7][5] = {0.000095203, 0.000095204, 0.000104141, 0.000099331, 0.000099331,
                 0.000151284, 0.000148333, 0.000151814, 0.000147953, 0.000143505,
                 0.000197612, 0.000196251, 0.000191317, 0.000195112, 0.000203217,
                 0.000156319, 0.000157184, 0.000163081, 0.000157349, 0.000162052,
                 0.00014439 , 0.000153818, 0.000170083, 0.000160573, 0.000139122,
                 0.000199936, 0.000236463, 0.000262118, 0.000240549, 0.000199932,
                 0.000226558, 0.000296774, 0.00032661 , 0.000292281, 0.000226592};

Double_t HMatrixParticle::GetResolution(Int_t iSetup)
{
   //Double_t theta = TMath::RadToDeg()*(pPart->Theta());  // polar angle in degree
   //Double_t phi = 90.-TMath::RadToDeg()*(pPart->Phi());  // azimuthal angle in degree
   //Double_t mass = pPart->M()*1000.;
   //Double_t p = (pPart->Vect()).Mag()*1000.;
   //if (phi<0.) phi += 360.;                // TLorentzVector uses ATan2(x,y)
   if(theta <= 15 || theta > 85) return 0.1;
   Int_t thetaBin = int((theta-15)/10);
   Int_t phiBin = int((phi - int(phi/60)*60)/12);
   Double_t c1, c2, res;
   switch(iSetup)
      {
      case 2:
     c1 = detkick[thetaBin][phiBin];
     res = c1*mom*mom;
     break;

      case 3:
     c1 = detrk3[thetaBin][phiBin];
     c2 = multscrk3[thetaBin][phiBin];
     res = sqrt(c1*c1*mom*mom*mom*mom + c2*c2*(mom*mom + invmass*invmass));
     break;

      case 4:
     c1 = detrk4[thetaBin][phiBin];
     c2 = multscrk4[thetaBin][phiBin];
     res = sqrt(c1*c1*mom*mom*mom*mom + c2*c2*(mom*mom + invmass*invmass));
     break;

      default:
     return 0.1;
      }
   return res/1000.;
}
//=======================================================

void HMatrixParticle::Smear(int n) {
  // smearing of momentum due to the HADES resolution
  // n == 2 : MDC I, II
  // n == 3 : MDC I, II, III
  // n == 4 : MDC I, II, III, IV

  Double_t sin_theta = TMath::Sin(theta*deg2rad);
  Double_t gamma;
  switch (n)
  {
     case 2: gamma = 0.01*(mom/1000.)*(2.0 + 13.63 * mom/1000.);
             break;
     case 3: gamma = 0.01*(mom/1000.)*(1.0 + 3.6 * mom/1000.);
             break;
     case 4: gamma = 0.01*(mom/1000.)*(0.5 * 1.36 * mom/1000.);
             break;
     default: gamma = 0.1;
              break;
  }
  gamma = GetResolution(n);

  if (TMath::Abs(sin_theta)<1.e-6) sin_theta = (sin_theta>0.) ? 1.e-6 : -1.e-6;
  Double_t smeared_mom = TMath::Abs( REngine.Gaus(mom/1000., gamma ) );
  Double_t smeared_theta = REngine.Gaus(theta*deg2rad, 0.0023 );
  Double_t smeared_phi = ( REngine.Gaus(sin_theta*phi*deg2rad, 0.0023 ) )/sin_theta;
  Double_t theta_check = smeared_theta-2.*pi*int(smeared_theta/(2.*pi));
  if (theta_check>pi||theta_check<0.) smeared_phi -= pi;
  mom = smeared_mom * 1000.;
  theta = smeared_theta * rad2deg;
  phi = smeared_phi * rad2deg;
  // -------------------------- recalculate momx, momy, momz -------------
  momx = mom * TMath::Sin(theta*deg2rad) * TMath::Cos(phi*deg2rad);
  momy = mom * TMath::Sin(theta*deg2rad) * TMath::Sin(phi*deg2rad);
  momz = mom * TMath::Cos(theta*deg2rad);
}


