// This macro filters data
// Date: 17.11.05 by WP

#include <TROOT.h>
#include <TRandom.h>
#include <TLorentzVector.h>
#include <string>

#include "hcategory.h"
#include "hgeantkine.h"


// ****************************************************************************
class HMatrixParticle: public TNamed
{

public:

   HMatrixParticle();
   HMatrixParticle(Int_t part, const char* name);
   ~HMatrixParticle();

   void setId(Int_t val);
   void setXYZmom(Float_t px, const Float_t py, const Float_t pz);
   void setMomThetaPhi(Float_t p, const Float_t th, const Float_t ph);
   void setMomtRapidity(Float_t pt, const Float_t y);
   void setParMedMech(Int_t par, const Int_t med, const Int_t mech);
   void setGenInfo(Float_t g0, const Float_t g1, const Float_t g2);
   void setParGparGGparId(Int_t parid, const Int_t gparid, const Int_t ggparid);
   void setAcc(Float_t ac);
   void setEff(Float_t ef);

   const char* getName() const;
   Int_t getId() const;
   Float_t getAcc() const;
   Float_t getEff() const;
   Float_t getMass() const;
   Float_t getMomx() const;
   Float_t getMomy() const;
   Float_t getMomz() const;
   Float_t getMom() const;
   Float_t getPt() const;
   Float_t getY() const;
   Float_t getTheta() const;
   Float_t getPhi() const;
   Int_t getParent() const;
   Int_t getParentId() const;
   Int_t getGrandparentId() const;
   Int_t getGrandgrandparentId() const;
   Int_t getMedium() const;
   Int_t getMechanism() const;
   Float_t getGenInfo() const;
   Float_t getGenInfo1() const;
   Float_t getGenInfo2() const;

   TLorentzVector* getLVector();

   Bool_t getUrQMDcondition() const;
   Bool_t getPLUTOcondition() const;
   void Smear(int n = 2);
   Double_t GetResolution(Int_t iSetup);
   Bool_t operator=(HGeantKine& hit);

private:

   std::string myName;
   Int_t id;
   Float_t mom, theta, phi;
   Float_t momt, momx, momy, momz, rapidity, invmass;
   Int_t parentid, grandparentid, grandgrandparentid, parent, medium, mechanism;
   Float_t geninfo, geninfo1, geninfo2;
   TLorentzVector vPart;

   Float_t acc, eff;

   HGeomVector vec;

   Double_t pi, rad2deg, deg2rad;
   TRandom REngine;

ClassDef(HMatrixParticle, 0)  //
};
// ****************************************************************************


