#include "hmissingfactory.h"
#include "hpidgeanttrackset.h"
#include "hgeantkine.h" 
#include "PParticle.h"
#include <stdexcept>
#include <TMath.h>
#include <TLorentzVector.h>

using namespace std;

// ****************************************************************************
ClassImp(HMissingFactory)

//-----------------------------------------------------------------------------
HMissingFactory::HMissingFactory(HParticlePool& ref, std::string name) : HInvariantFactory( ref, name )
{
}

//-----------------------------------------------------------------------------
HMissingFactory::~HMissingFactory()
{
}



//-----------------------------------------------------------------------------
void HMissingFactory::combine()
{
   int i1, i2, i3, i4, repetitionNr, combinitaionNr;
   EParticle pid;

   particleSet.clear();
   count();
   particleSet.resize( number );
   for (int i = 0; i < number; ++i)
   {
      if (0 != refPool.ptrTarget)
      {
         particleSet[i] += *refPool.ptrTarget;
      }
      else
      {
         throw std::invalid_argument("No target defined and missing mass cannot be calculated!");
      }
      if (0 != refPool.ptrBeam)
      {
         particleSet[i] += *refPool.ptrBeam;
      }
      else
      {
         throw std::invalid_argument("No beam defined and missing mass cannot be calculated!");
      }
   }
      
   
   // here all HParticle objects from HParticlePool should be combined according to pattern from mapN

   MultiParticleIter mpIter;
   MultiParticleIter mpBeginIter;
   for (particleNumIter = particleComb.begin(); particleNumIter != particleComb.end(); ++particleNumIter)
   // loop over all particle species
   {
      pid = particleNumIter->first;
      combinitaionNr = particleNumIter->second;
      if (combinitaionNr > 0) // for given particle: if number of combinations > 0
      {
            repetitionNr = number / combinitaionNr;
            for (int k=0; k < combinitaionNr; ++k) // all combinations
            {
               switch (particleNum[ pid ])
               {
                   case 1: i1 = ptrFH->map1[ 10 + refPool.particleNum[ pid ] ][ k ][0];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           break;

                   case 2: i1 = ptrFH->map2[ 20 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map2[ 20 + refPool.particleNum[ pid ] ][ k ][1];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           break;

                   case 3: i1 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][1];
                           i3 = ptrFH->map3[ 30 + refPool.particleNum[ pid ] ][ k ][2];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i3--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           break;

                   case 4: i1 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][0];
                           i2 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][1];
                           i3 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][2];
                           i4 = ptrFH->map4[ 40 + refPool.particleNum[ pid ] ][ k ][3];
                           mpBeginIter = refPool.lower_bound( pid );
                           mpIter = mpBeginIter;
                           while (i1--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i2--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i3--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           mpIter = mpBeginIter;
                           while (i4--) ++mpIter;
                           for (int j=0; j < repetitionNr; ++j) // number of repetitions of combinations
                              particleSet [ k + j*combinitaionNr ] -= mpIter->second;
                           break;

               }
            }
      }
   }

}

