#ifndef HPARTICLE_H
#define HPARTICLE_H

#include <TROOT.h>
#include <TRandom.h>
#include <TLorentzVector.h>
#include <string>
#include <iostream>
#include "hcategory.h"
#include "hgeantkine.h"

//class HGeantKine;
class PParticle;

// ****************************************************************************
class HParticle: public TObject
{

public:

   HParticle();
   HParticle(Int_t part, const Char_t* name="");
   HParticle(const Char_t* name, double px, double py, double pz);
   ~HParticle() {}

   void setId(Long_t id) { this->id=id; }
   void setName(const Char_t* name) { this->name=name; }
   void setIdName(Int_t id, const Char_t* name="") { this->id=id; this->name=name; }

   void setAcc(Double_t acc) { this->acc=acc; }
   void setEff(Double_t eff) { this->eff=eff; }

   void setPxPyPzM(Double_t px=0., Double_t py=0., Double_t pz=0., Double_t mass=0.);
   void setPxPyPz(Double_t px=0., Double_t py=0., Double_t pz=0.);

   void setXYZ(Double_t x, Double_t y, Double_t z) { vertex.x=x; vertex.y=y; vertex.z=z; } 
   void setParentTrack(Int_t ptrack) { orig.parentTrack=ptrack; }
   void setMedium(Int_t medium) { orig.medium=medium; }
   void setMechanism(Int_t mechanism) { orig.mechanism=mechanism; }
   void setParMedMech(Int_t par, Int_t med, Int_t mech) { orig.parentTrack=par; orig.medium=med; orig.mechanism=mech; }
   void setGenInfo(Double_t g0, Double_t g1, Double_t g2) { orig.genInfo=g0; orig.genInfo1=g1; orig.genInfo2=g2; }
   void setPar(Int_t parid) { orig.parentId=parid; }
   void setParGparGGparId(Int_t parid, const Int_t gparid, const Int_t ggparid) {
	   orig.parentId=parid; orig.grandParentId=gparid; orig.grandGrandParentId=ggparid; }

   Long_t getId() const { return id; }
   const Char_t* getName() const { return name.c_str(); }
   Double_t getMass() const { return TMath::Abs(vPart.M()); }
   Double_t getMass2() const { return TMath::Abs(vPart.M2()); }
   Double_t getE() const { return vPart.E(); }
   
   Double_t getAcc() const { return acc; }
   Double_t getEff() const { return eff; }

   Double_t getX() const { return vertex.x; }
   Double_t getY() const { return vertex.y; }
   Double_t getZ() const { return vertex.z; }

   Double_t getP() const { return vPart.P(); }
   Double_t getPt() const { return vPart.Pt(); }
   Double_t getPx() const { return vPart.Px(); }
   Double_t getPy() const { return vPart.Py(); }
   Double_t getPz() const { return vPart.Pz(); }
   Double_t getRapidity() const { return vPart.Rapidity(); }
   Double_t getTheta(std::string unit = "rad") const;
   Double_t getPhi(std::string unit = "rad", std::string sector = "") const;

   Int_t getMedium() const { return orig.medium; }
   Int_t getMechanism() const { return orig.mechanism; }
   Double_t getGenInfo() const { return orig.genInfo; }
   Double_t getGenInfo1() const { return orig.genInfo1; }
   Double_t getGenInfo2() const { return orig.genInfo2; }
   Int_t getParentTrack() const { return orig.parentTrack; }
   Int_t getParentId() const { return orig.parentId; }
   Int_t getGrandParentId() const { return orig.grandParentId; }
   Int_t getGrandGrandParentId() const { return orig.grandGrandParentId; }

   const TLorentzVector* getLVector() const { return &vPart; }

   //Bool_t isPrimary() const;

   HParticle(const HParticle& src);
   HParticle& operator=(const HParticle& src);
   HParticle(const PParticle& hit);  
   HParticle(HGeantKine& hit); // not const because of not const functions in HGeantKine !!!
   HParticle& operator+=(const HParticle& part);
   HParticle& operator-=(const HParticle& part);
   HParticle  operator+(const HParticle& part);
   HParticle  operator-(const HParticle& part);

protected:

   Long_t id;
   std::string name;

   Double_t acc, eff;

   struct HOrigin {
      HOrigin() : parentId(0), grandParentId(0), grandGrandParentId(0),
		  parentTrack(0), medium(0), mechanism(0),
		  genInfo(0.), genInfo1(0.), genInfo2(0.) {}
      ~HOrigin() {}
      Int_t parentId, grandParentId, grandGrandParentId; 
      Int_t parentTrack, medium, mechanism;
      Float_t genInfo, genInfo1, genInfo2;
   } orig;

   struct HVertex {
      HVertex() : x(0.), y(0.), z(0.) {}
      ~HVertex() {}
      void reset() { x=0.; y=0.; z=0.; }
      Double_t x, y, z;
   } vertex;
   
   TLorentzVector vPart;

   HGeomVector vec;

   double shiftPhi(double phi) const;

ClassDef(HParticle, 0)  //
};
// ****************************************************************************

#endif // HPARTICLE_H
