#ifndef HPARTICLEPOOL_H
#define HPARTICLEPOOL_H

#include <TROOT.h>
#include <TRandom.h>
#include <TLorentzVector.h>
#include <string>
#include <list>
#include <functional>
#include <algorithm>

#include <iostream>

#include "hcategory.h"
#include "hgeantkine.h"

#include "hparticle.h"
#include "hnparticle.h"
#include "hparticles.h"
#include "hsourcefile.h"
#include "houtputfile.h"
#include "hfilter.h"
#include "hinput.h"
#include "hntuple.h"


using namespace FilteringUtil;
class HPoolFactory;


// ****************************************************************************
class HParticlePool 
{

   class HEntity {
    public:
      HEntity(HParticlePool& ref, std::string name, std::string type);
      ~HEntity();
      std::string name;
      HPoolFactory *ptrPF;
      HNtuple *ptrHN;
   };

   struct IsName : std::binary_function<HEntity*, const char*, bool> {
      bool operator()(HEntity* ptrE, const char* name) const {
         return ( std::string(name) == ptrE->name );
      }
   };

   struct DelListObj {
      void operator()(HEntity* ptr) {
         delete ptr;
      }
   };



   friend class HPoolFactory;
   friend class HInvariantFactory;
   friend class HMissingFactory;

public:

   HParticlePool();
   HParticlePool(const HFile& src);
   HParticlePool(const HFile& src, HFile& out);
   HParticlePool(const HFile& src, HFile& out, const HFilter& fil);
   ~HParticlePool();

   void setSource(const HFile& ref);
   void setOutput(HFile& ref);
   void setFilter(const HFilter& ref);

   void target(const char* part, double px, double py, double pz);
   void beam(const char* part, double px, double py, double pz);

   void invariant(const char* name);
   void missing(const char* name);
   void add(const char* name, EParticle p1);
   void add(const char* name, EParticle p1, EParticle p2);
   void add(const char* name, EParticle p1, EParticle p2, EParticle p3);
   void add(const char* name, EParticle p1, EParticle p2, EParticle p3, EParticle p4);
   void add(const char* name, const char* p1);
   void add(const char* name, const char* p1, const char* p2);
   void add(const char* name, const char* p1, const char* p2, const char* p3);
   void add(const char* name, const char* p1, const char* p2, const char* p3, const char* p4);

   void loop();
   void write();


protected:

   int count( EParticle n );
   MultiParticleIterPair equal_range( EParticle n );
   MultiParticleIter lower_bound( EParticle n );
   MultiParticleIter upper_bound( EParticle n );

   MultiParticle particle;
   ParticleNum particleNum;
   MultiParticleIter particleIter;
   MultiParticleIterPair particleIterPair;

   void dump(HNParticle& ptrP, HNtuple& ntuple);

   HParticle *ptrTarget;
   HParticle *ptrBeam;

   const HFile *pSourceFile;
   HFile *pOutputFile;

   HInput *pInput;

   const HFilter *pFilter;

   std::list<HEntity*> objectList;

   
   bool isObject(const char* name);
   HEntity* getObject(const char* name);

   EParticle convertName(const char* name);
   EParticle convertId(int n);


ClassDef(HParticlePool, 0)  //
};
// ****************************************************************************

#endif // HPARTICLEPOOL_H
